@@ -2,23 +2,23 @@
  * Copyright (c) 1996-1997 Sam Leffler
  * Copyright (c) 1996 Pixar
  *
- * Permission to use, copy, modify, distribute, and sell this software and 
+ * Permission to use, copy, modify, distribute, and sell this software and
  * its documentation for any purpose is hereby granted without fee, provided
  * that (i) the above copyright notices and this permission notice appear in
  * all copies of the software and related documentation, and (ii) the names of
  * Pixar, Sam Leffler and Silicon Graphics may not be used in any advertising or
  * publicity relating to the software without the specific, prior written
  * permission of Pixar, Sam Leffler and Silicon Graphics.
- * 
- * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
- * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
- * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
- * 
+ *
+ * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
+ * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
+ *
  * IN NO EVENT SHALL PIXAR, SAM LEFFLER OR SILICON GRAPHICS BE LIABLE FOR
  * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
  * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
- * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
+ * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF
+ * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
  * OF THIS SOFTWARE.
  */
 
@@ -31,10 +31,10 @@
  * Contributed by Dan McCoy.
  *
  * PixarLog film support uses the TIFF library to store companded
- * 11 bit values into a tiff file, which are compressed using the 
- * zip compressor.  
+ * 11 bit values into a tiff file, which are compressed using the
+ * zip compressor.
  *
- * The codec can take as input and produce as output 32-bit IEEE float values 
+ * The codec can take as input and produce as output 32-bit IEEE float values
  * as well as 16-bit or 8-bit unsigned integer values.
  *
  * On writing any of the above are converted into the internal
@@ -48,7 +48,7 @@
  * than the human eye can perceive with extra room to allow for
  * error introduced by further image computation.  As with any quantized
  * color format, it is possible to perform image calculations which
- * expose the quantization error. This format should certainly be less 
+ * expose the quantization error. This format should certainly be less
  * susceptable to such errors than standard 8-bit encodings, but more
  * susceptable than straight 16-bit or 32-bit encodings.
  *
@@ -80,14 +80,13 @@
  * The codec also handle byte swapping the encoded values as necessary
  * since the library does not have the information necessary
  * to know the bit depth of the raw unencoded buffer.
- * 
+ *
  */
 
 #include "tif_predict.h"
 #undef EXPORT
 #include "zlib.h"
 
-#include <stdio.h>
 #include <assert.h>
 #include <math.h>
 #include "imgInt.h"
@@ -166,7 +165,7 @@ horizontalAccumulateF(wp, n, stride, op, ToLinearF)
     uint16 *wp;
     int n;
     int stride;
-    float *op; 
+    float *op;
     float *ToLinearF;
 {
     register unsigned int  cr, cg, cb, ca, mask;
@@ -352,7 +351,7 @@ horizontalAccumulate16(wp, n, stride, op, ToLinear16)
     }
 }
 
-/* 
+/*
  * Returns the log encoded 11-bit values with the horizontal
  * differencing undone.
  */
@@ -390,7 +389,7 @@ horizontalAccumulate11(wp, n, stride, op)
 		op[1] = (cg += wp[1]) & mask;
 		op[2] = (cb += wp[2]) & mask;
 		op[3] = (ca += wp[3]) & mask;
-	    } 
+	    }
 	} else {
 	    REPEAT(stride, *op = *wp&mask; wp++; op++)
 	    n -= stride;
@@ -532,7 +531,7 @@ horizontalAccumulate8abgr(wp, n, stride, op, ToLinear8)
 typedef	struct {
 	TIFFPredictorState	predict;
 	z_stream		stream;
-	uint16			*tbuf; 
+	uint16			*tbuf;
 	uint16			stride;
 	int			state;
 	int			user_datafmt;
@@ -548,7 +547,7 @@ typedef	struct {
 	uint16  *FromLT2;
 	uint16  *From14; /* Really for 16-bit data, but we shift down 2 */
 	uint16  *From8;
-	
+
 } PixarLogState;
 
 static int
@@ -562,7 +561,7 @@ PixarLogMakeTables(sp)
  *    11-bit companded representation.  The 11-bit representation has two
  *    distinct regions.  A linear bottom end up through .018316 in steps
  *    of about .000073, and a region of constant ratio up to about 25.
- *    These floating point numbers are stored in the main table ToLinearF. 
+ *    These floating point numbers are stored in the main table ToLinearF.
  *    All other tables are derived from this one.  The tables (and the
  *    ratios) are continuous at the internal seam.
  */
@@ -578,7 +577,7 @@ PixarLogMakeTables(sp)
     uint16  *From14; /* Really for 16-bit data, but we shift down 2 */
     uint16  *From8;
 
-    c = log(RATIO);	
+    c = log(RATIO);
     nlin = 1./c;	/* nlin must be an integer */
     c = 1./nlin;
     b = exp(-c*ONE);	/* multiplicative scale factor [b*exp(c*ONE) = 1] */
@@ -727,13 +726,13 @@ PixarLogSetupDecode(tif)
 
 	sp->stride = (td->td_planarconfig == PLANARCONFIG_CONTIG ?
 	    td->td_samplesperpixel : 1);
-	sp->tbuf = (uint16 *) ImgTIFFmalloc(sp->stride * 
+	sp->tbuf = (uint16 *) ImgTIFFmalloc(sp->stride *
 		td->td_imagewidth * td->td_rowsperstrip * sizeof(uint16));
 	if (sp->user_datafmt == PIXARLOGDATAFMT_UNKNOWN)
 		sp->user_datafmt = PixarLogGuessDataFmt(td);
 	if (sp->user_datafmt == PIXARLOGDATAFMT_UNKNOWN) {
-		ImgTIFFError(module, 
-			"PixarLog compression can't handle bits depth/data format combination (depth: %d)", 
+		ImgTIFFError(module,
+			"PixarLog compression can't handle bits depth/data format combination (depth: %d)",
 			td->td_bitspersample);
 		return (0);
 	}
@@ -884,7 +883,7 @@ PixarLogDecode(tif, op, occ, s)
 			break;
 		default:
 			ImgTIFFError(tif->tif_name,
-				  "PixarLogDecode: unsupported bits/sample: %d", 
+				  "PixarLogDecode: unsupported bits/sample: %d",
 				  td->td_bitspersample);
 			return (0);
 		}
@@ -907,7 +906,7 @@ PixarLogSetupEncode(tif)
 
 	sp->stride = (td->td_planarconfig == PLANARCONFIG_CONTIG ?
 	    td->td_samplesperpixel : 1);
-	sp->tbuf = (uint16 *) ImgTIFFmalloc(sp->stride * 
+	sp->tbuf = (uint16 *) ImgTIFFmalloc(sp->stride *
 		td->td_imagewidth * td->td_rowsperstrip * sizeof(uint16));
 	if (sp->user_datafmt == PIXARLOGDATAFMT_UNKNOWN)
 		sp->user_datafmt = PixarLogGuessDataFmt(td);
@@ -1173,17 +1172,17 @@ PixarLogEncode(tif, bp, cc, s)
 	for (i = 0, up = sp->tbuf; i < n; i += llen, up += llen) {
 		switch (sp->user_datafmt)  {
 		case PIXARLOGDATAFMT_FLOAT:
-			horizontalDifferenceF((float *)bp, llen, 
+			horizontalDifferenceF((float *)bp, llen,
 				sp->stride, up, sp->FromLT2);
 			bp += llen * sizeof(float);
 			break;
 		case PIXARLOGDATAFMT_16BIT:
-			horizontalDifference16((uint16 *)bp, llen, 
+			horizontalDifference16((uint16 *)bp, llen,
 				sp->stride, up, sp->From14);
 			bp += llen * sizeof(uint16);
 			break;
 		case PIXARLOGDATAFMT_8BIT:
-			horizontalDifference8((unsigned char *)bp, llen, 
+			horizontalDifference8((unsigned char *)bp, llen,
 				sp->stride, up, sp->From8);
 			bp += llen * sizeof(unsigned char);
 			break;
@@ -1194,7 +1193,7 @@ PixarLogEncode(tif, bp, cc, s)
 			return 0;
 		}
 	}
- 
+
 	sp->stream.next_in = (unsigned char *) sp->tbuf;
 	sp->stream.avail_in = n * sizeof(uint16);
 
@@ -1379,6 +1378,23 @@ PixarLogVGetField(tif, tag, ap)
     return (1);
 }
 
+static voidpf
+PixarLogAlloc(opaque, items, size)
+    voidpf opaque;
+    uInt items;
+    uInt size;
+{
+    return (voidpf) ImgTIFFmalloc((tsize_t)(items * size));
+}
+
+static void
+PixarLogFree(opaque, address)
+    voidpf opaque;
+    voidpf address;
+{
+    ImgTIFFfree((tdata_t) address);
+}
+
 static const TIFFFieldInfo pixarlogFieldInfo[] = {
     {TIFFTAG_PIXARLOGDATAFMT,0,0,TIFF_ANY,  FIELD_PSEUDO,FALSE,FALSE,""},
     {TIFFTAG_PIXARLOGQUALITY,0,0,TIFF_ANY,  FIELD_PSEUDO,FALSE,FALSE,""}
@@ -1408,6 +1424,8 @@ ImgInitTIFFpixar(handle, scheme)
 	sp = (PixarLogState*) tif->tif_data;
 	memset(sp, 0, sizeof (*sp));
 	sp->stream.data_type = Z_BINARY;
+	sp->stream.zalloc = PixarLogAlloc;
+	sp->stream.zfree = PixarLogFree;
 	sp->user_datafmt = PIXARLOGDATAFMT_UNKNOWN;
 
 	/*
@@ -1438,13 +1456,13 @@ ImgInitTIFFpixar(handle, scheme)
 	sp->quality = Z_DEFAULT_COMPRESSION; /* default comp. level */
 	sp->state = 0;
 
-	/* we don't wish to use the predictor, 
+	/* we don't wish to use the predictor,
 	 * the default is none, which predictor value 1
 	 */
 	(void) ImgTIFFPredictorInit(tif);
 
 	/*
-	 * build the companding tables 
+	 * build the companding tables
 	 */
 	PixarLogMakeTables(sp);
 
