@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkText.h,v 1.12 2002/08/05 04:30:40 dgp Exp $
+ * RCS: @(#) $Id: tkText.h,v 1.13 2002/12/27 23:43:38 davygrvy Exp $
  */
 
 #ifndef _TKTEXT
@@ -34,7 +34,7 @@
  * file:
  */
 
-typedef struct TkTextBTree *TkTextBTree;
+typedef struct TkTextBTree_ *TkTextBTree;
 
 /*
  * The data structure below defines a single line of text (from newline
@@ -347,7 +347,7 @@ typedef struct TkTextTag {
 				 * baseline of line.  Used for superscripts
 				 * and subscripts.  Only valid if
 				 * offsetString is non-NULL. */
-    Arg overstrikeString;	/* -overstrike option string (malloc-ed).
+    Tcl_Obj * overstrikeString;	/* -overstrike option string (malloc-ed).
 				 * NULL means option not specified. */
     int overstrike;		/* Non-zero means draw horizontal line through
 				 * middle of text.  Only valid if
@@ -371,12 +371,12 @@ typedef struct TkTextTag {
     int spacing3;		/* Extra spacing below last display
 				 * line for text line.  Only valid if
 				 * spacing3String is non-NULL. */
-    Arg tabString;		/* -tabs option string (malloc-ed).
+    Tcl_Obj * tabString;		/* -tabs option string (malloc-ed).
 				 * NULL means option not specified. */
     struct TkTextTabArray *tabArrayPtr;
 				/* Info about tabs for tag (malloc-ed)
 				 * or NULL.  Corresponds to tabString. */
-    Arg underlineString;	/* -underline option string (malloc-ed).
+    Tcl_Obj * underlineString;	/* -underline option string (malloc-ed).
 				 * NULL means option not specified. */
     int underline;		/* Non-zero means draw underline underneath
 				 * text.  Only valid if underlineString is
@@ -386,16 +386,14 @@ typedef struct TkTextTag {
 				 * TEXT_WRAPMODE_NONE, TEXT_WRAPMODE_WORD,
 				 * or TEXT_WRAPMODE_NULL to use wrapmode for
 				 * whole widget. */
-    Arg elideString;		/* -elide option string (malloc-ed).
+    Tcl_Obj * elideString;		/* -elide option string (malloc-ed).
 				 * NULL means option not specified. */
-    int elide;		/* Non-zero means text is elided.
-				 * Only valid if elideString is non-NULL. */
+    int elide;			/* Non-zero means that data under this tag
+				 * should not be displayed. */
     int affectsDisplay;		/* Non-zero means that this tag affects the
 				 * way information is displayed on the screen
 				 * (so need to redisplay if tag changes). */
-    Tk_State state;		/* Must be TK_STATE_NULL, TK_STATE_NORMAL,
-				 * TK_STATE_HIDDEN or TK_STATE_DISABLED. */
-    Arg userData;		/* arbitary user data */
+    Tcl_Obj * userData;		/* arbitary user data */
 } TkTextTag;
 
 #define TK_TAG_AFFECTS_DISPLAY	0x1
@@ -534,7 +532,7 @@ typedef struct TkText {
 				 * for the same text line. */
     int spacing3;		/* Default extra spacing below last display
 				 * line for each text line. */
-    Arg tabOptionString;	/* Value of -tabs option string (malloc'ed). */
+    Tcl_Obj * tabOptionString;	/* Value of -tabs option string (malloc'ed). */
     TkTextTabArray *tabArrayPtr;
 				/* Information about tab stops (malloc'ed).
 				 * NULL means perform default tabbing
