@@ -223,7 +223,7 @@ Tk_GetVisual(interp, tkwin, arg, depthPtr, colormapPtr)
 	if (*p == 0) {
 	    template.depth = 10000;
 	} else {
-            Arg temp = NULL;
+            Tcl_Obj * temp = NULL;
             LangSetString(&temp,p);
 	    if (Tcl_GetIntFromObj(interp, temp, &template.depth) != TCL_OK) {
                 LangFreeArg(temp,TCL_STATIC);
