@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- *  RCS: @(#) $Id: tclUtil.c,v 1.34 2002/08/05 15:01:05 dgp Exp $
+ *  RCS: @(#) $Id: tclUtil.c,v 1.36.2.4 2003/08/27 21:31:53 dgp Exp $
  */
 
 #include "tkPort.h"
@@ -924,7 +924,7 @@ Tcl_Backslash(src, readPtr)
     Tcl_UniChar ch;
 
     Tcl_UtfBackslash(src, readPtr, buf);
-    Tcl_UtfToUniChar(buf, &ch);
+    TclUtfToUniChar(buf, &ch);
     return (char) ch;
 }
 
@@ -1191,7 +1191,7 @@ Tcl_StringCaseMatch(string, pattern, nocase)
 				 * characters. */
     int nocase;			/* 0 for case sensitive, 1 for insensitive */
 {
-    int p;
+    int p, charLen;
     CONST char *pstart = pattern;
     Tcl_UniChar ch1, ch2;
 
@@ -1227,9 +1227,17 @@ Tcl_StringCaseMatch(string, pattern, nocase)
 	    if (p == '\0') {
 		return 1;
 	    }
-	    Tcl_UtfToUniChar(pattern, &ch2);
-	    if (nocase) {
-		ch2 = Tcl_UniCharToLower(ch2);
+	    /*
+	     * This is a special case optimization for single-byte utf.
+	     */
+	    if (UCHAR(*pattern) < 0x80) {
+		ch2 = (Tcl_UniChar)
+		    (nocase ? tolower(UCHAR(*pattern)) : UCHAR(*pattern));
+	    } else {
+		Tcl_UtfToUniChar(pattern, &ch2);
+		if (nocase) {
+		    ch2 = Tcl_UniCharToLower(ch2);
+		}
 	    }
 	    while (1) {
 		/*
@@ -1240,7 +1248,7 @@ Tcl_StringCaseMatch(string, pattern, nocase)
 		if ((p != '[') && (p != '?') && (p != '\\')) {
 		    if (nocase) {
 			while (*string) {
-			    int charLen = Tcl_UtfToUniChar(string, &ch1);
+			    charLen = TclUtfToUniChar(string, &ch1);
 			    if (ch2==ch1 || ch2==Tcl_UniCharToLower(ch1)) {
 				break;
 			    }
@@ -1253,7 +1261,7 @@ Tcl_StringCaseMatch(string, pattern, nocase)
 			 * compare each time is non-constant.
 			 */
 			while (*string) {
-			    int charLen = Tcl_UtfToUniChar(string, &ch1);
+			    charLen = TclUtfToUniChar(string, &ch1);
 			    if (ch2 == ch1) {
 				break;
 			    }
@@ -1267,7 +1275,7 @@ Tcl_StringCaseMatch(string, pattern, nocase)
 		if (*string == '\0') {
 		    return 0;
 		}
-		string += Tcl_UtfToUniChar(string, &ch1);
+		string += TclUtfToUniChar(string, &ch1);
 	    }
 	}
 
@@ -1278,7 +1286,7 @@ Tcl_StringCaseMatch(string, pattern, nocase)
 
 	if (p == '?') {
 	    pattern++;
-	    string += Tcl_UtfToUniChar(string, &ch1);
+	    string += TclUtfToUniChar(string, &ch1);
 	    continue;
 	}
 
@@ -1292,26 +1300,45 @@ Tcl_StringCaseMatch(string, pattern, nocase)
 	    Tcl_UniChar startChar, endChar;
 
 	    pattern++;
-	    string += Tcl_UtfToUniChar(string, &ch1);
-	    if (nocase) {
-		ch1 = Tcl_UniCharToLower(ch1);
+	    if (UCHAR(*string) < 0x80) {
+		ch1 = (Tcl_UniChar)
+		    (nocase ? tolower(UCHAR(*string)) : UCHAR(*string));
+		string++;
+	    } else {
+		string += Tcl_UtfToUniChar(string, &ch1);
+		if (nocase) {
+		    ch1 = Tcl_UniCharToLower(ch1);
+		}
 	    }
 	    while (1) {
 		if ((*pattern == ']') || (*pattern == '\0')) {
 		    return 0;
 		}
-		pattern += Tcl_UtfToUniChar(pattern, &startChar);
-		if (nocase) {
-		    startChar = Tcl_UniCharToLower(startChar);
+		if (UCHAR(*pattern) < 0x80) {
+		    startChar = (Tcl_UniChar)
+			(nocase ? tolower(UCHAR(*pattern)) : UCHAR(*pattern));
+		    pattern++;
+		} else {
+		    pattern += Tcl_UtfToUniChar(pattern, &startChar);
+		    if (nocase) {
+			startChar = Tcl_UniCharToLower(startChar);
+		    }
 		}
 		if (*pattern == '-') {
 		    pattern++;
 		    if (*pattern == '\0') {
 			return 0;
 		    }
-		    pattern += Tcl_UtfToUniChar(pattern, &endChar);
-		    if (nocase) {
-			endChar = Tcl_UniCharToLower(endChar);
+		    if (UCHAR(*pattern) < 0x80) {
+			endChar = (Tcl_UniChar)
+			    (nocase ? tolower(UCHAR(*pattern))
+				    : UCHAR(*pattern));
+			pattern++;
+		    } else {
+			pattern += Tcl_UtfToUniChar(pattern, &endChar);
+			if (nocase) {
+			    endChar = Tcl_UniCharToLower(endChar);
+			}
 		    }
 		    if (((startChar <= ch1) && (ch1 <= endChar))
 			    || ((endChar <= ch1) && (ch1 <= startChar))) {
@@ -1353,8 +1380,8 @@ Tcl_StringCaseMatch(string, pattern, nocase)
 	 * bytes of each string match.
 	 */
 
-	string  += Tcl_UtfToUniChar(string, &ch1);
-	pattern += Tcl_UtfToUniChar(pattern, &ch2);
+	string  += TclUtfToUniChar(string, &ch1);
+	pattern += TclUtfToUniChar(pattern, &ch2);
 	if (nocase) {
 	    if (Tcl_UniCharToLower(ch1) != Tcl_UniCharToLower(ch2)) {
 		return 0;
@@ -1717,8 +1744,7 @@ Tcl_DStringGetResult(interp, dsPtr)
 
     dsPtr->length = strlen(iPtr->result);
     if (iPtr->freeProc != NULL) {
-	if ((iPtr->freeProc == TCL_DYNAMIC)
-		|| (iPtr->freeProc == (Tcl_FreeProc *) free)) {
+	if (iPtr->freeProc == TCL_DYNAMIC) {
 	    dsPtr->string = iPtr->result;
 	    dsPtr->spaceAvl = dsPtr->length+1;
 	} else {
@@ -1839,11 +1865,16 @@ Tcl_PrintDouble(interp, value, dst)
      * If the ASCII result looks like an integer, add ".0" so that it
      * doesn't look like an integer anymore.  This prevents floating-point
      * values from being converted to integers unintentionally.
+     * Check for ASCII specifically to speed up the function.
      */
 
     for (p = dst; *p != 0; ) {
-	p += Tcl_UtfToUniChar(p, &ch);
-	c = UCHAR(ch);
+	if (UCHAR(*p) < 0x80) {
+	    c = *p++;
+	} else {
+	    p += Tcl_UtfToUniChar(p, &ch);
+	    c = UCHAR(ch);
+	}
 	if ((c == '.') || isalpha(UCHAR(c))) {	/* INTL: ISO only. */
 	    return;
 	}
@@ -1967,45 +1998,67 @@ TclPrecTraceProc(clientData, interp, name1, name2, flags)
 int
 TclNeedSpace(start, end)
     CONST char *start;		/* First character in string. */
-    CONST char *end;			/* End of string (place where space will
+    CONST char *end;		/* End of string (place where space will
 				 * be added, if appropriate). */
 {
-    Tcl_UniChar ch;
-
     /*
      * A space is needed unless either
      * (a) we're at the start of the string, or
-     * (b) the trailing characters of the string consist of one or more
-     *     open curly braces preceded by a space or extending back to
-     *     the beginning of the string.
-     * (c) the trailing characters of the string consist of a space
-     *	   preceded by a character other than backslash.
      */
-
     if (end == start) {
 	return 0;
     }
+
+    /*
+     * (b) we're at the start of a nested list-element, quoted with an
+     *     open curly brace; we can be nested arbitrarily deep, so long
+     *     as the first curly brace starts an element, so backtrack over
+     *     open curly braces that are trailing characters of the string; and
+     */
+
     end = Tcl_UtfPrev(end, start);
-    if (*end != '{') {
-	Tcl_UtfToUniChar(end, &ch);
-	/*
-	 * Direct char comparison on next line is safe as it is with
-	 * a character in the ASCII subset, and so single-byte in UTF8.
-	 */
-	if (Tcl_UniCharIsSpace(ch) && ((end == start) || (end[-1] != '\\'))) {
-	    return 0;
-	}
-	return 1;
-    }
-    do {
+    while (*end == '{') {
 	if (end == start) {
 	    return 0;
 	}
 	end = Tcl_UtfPrev(end, start);
-    } while (*end == '{');
-    Tcl_UtfToUniChar(end, &ch);
-    if (Tcl_UniCharIsSpace(ch)) {
-	return 0;
+    }
+
+    /*
+     * (c) the trailing character of the string is already a list-element
+     *     separator (according to TclFindElement); that is, one of these
+     *     characters:
+     *     	\u0009	\t	TAB
+     *     	\u000A	\n	NEWLINE
+     *     	\u000B	\v	VERTICAL TAB
+     *     	\u000C	\f	FORM FEED
+     *     	\u000D	\r	CARRIAGE RETURN
+     *     	\u0020		SPACE
+     *     with the condition that the penultimate character is not a
+     *     backslash.
+     */
+
+    if (*end > 0x20) {
+	/*
+	 * Performance tweak.  All ASCII spaces are <= 0x20. So get
+	 * a quick answer for most characters before comparing against
+	 * all spaces in the switch below.
+	 *
+	 * NOTE: Remove this if other Unicode spaces ever get accepted
+	 * as list-element separators.
+	 */
+	return 1;
+    }
+    switch (*end) {
+	case ' ':
+        case '\t':
+        case '\n':
+        case '\r':
+        case '\v':
+        case '\f':
+	    if ((end == start) || (end[-1] != '\\')) {
+		return 0;
+	    }
     }
     return 1;
 }
@@ -2188,9 +2241,7 @@ TclGetIntForIndex(interp, objPtr, endValue, indexPtr)
 {
     char *bytes;
     int offset;
-#ifndef TCL_WIDE_INT_IS_LONG
     Tcl_WideInt wideOffset;
-#endif
 
     /*
      * If the object is already an integer, use it.
@@ -2205,16 +2256,14 @@ TclGetIntForIndex(interp, objPtr, endValue, indexPtr)
      * If the object is already a wide-int, and it is not out of range
      * for an integer, use it. [Bug #526717]
      */
-#ifndef TCL_WIDE_INT_IS_LONG
     if (objPtr->typePtr == &tclWideIntType) {
-	Tcl_WideInt wideOffset = objPtr->internalRep.wideValue;
+	TclGetWide(wideOffset,objPtr);
 	if (wideOffset >= Tcl_LongAsWide(INT_MIN)
 	    && wideOffset <= Tcl_LongAsWide(INT_MAX)) {
 	    *indexPtr = (int) Tcl_WideAsLong(wideOffset);
 	    return TCL_OK;
 	}
     }
-#endif /* TCL_WIDE_INT_IS_LONG */
 
     if (SetEndOffsetFromAny(NULL, objPtr) == TCL_OK) {
 	/*
@@ -2224,15 +2273,6 @@ TclGetIntForIndex(interp, objPtr, endValue, indexPtr)
 
 	*indexPtr = endValue + objPtr->internalRep.longValue;
 
-#ifdef TCL_WIDE_INT_IS_LONG
-    } else if (Tcl_GetIntFromObj(NULL, objPtr, &offset) == TCL_OK) {
-	/*
-	 * If the object can be converted to an integer, use that.
-	 */
-
-	*indexPtr = offset;
-
-#else /* !TCL_WIDE_INT_IS_LONG */
     } else if (Tcl_GetWideIntFromObj(NULL, objPtr, &wideOffset) == TCL_OK) {
 	/*
 	 * If the object can be converted to a wide integer, use
@@ -2251,7 +2291,6 @@ TclGetIntForIndex(interp, objPtr, endValue, indexPtr)
 	}
 	*indexPtr = offset;
 
-#endif /* TCL_WIDE_INT_IS_LONG */
     } else {
 	/*
 	 * Report a parse error.
@@ -2374,14 +2413,15 @@ SetEndOffsetFromAny(interp, objPtr)
 
     if (length <= 3) {
 	offset = 0;
-    } else if (bytes[3] == '-') {
+    } else if ((length > 4) && (bytes[3] == '-')) {
 	/*
-	 * This is our limited string expression evaluator
+	 * This is our limited string expression evaluator.  Pass everything
+	 * after "end-" to Tcl_GetInt, then reverse for offset.
 	 */
-	if (Tcl_GetInt(interp, bytes+3, &offset) != TCL_OK) {
+	if (Tcl_GetInt(interp, bytes+4, &offset) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-
+	offset = -offset;
     } else {
 	/*
 	 * Conversion failed.  Report the error.
